@@ -3,9 +3,11 @@
 #' Produce violin plot(s) of the given (grouped) values with enhanced annotation and colour per group. Includes customisation of colours for each aspect of the violin, boxplot, and separate violins. This supports input of data as a list or formula, being backwards compatible with \code{\link[vioplot]{vioplot}} (0.2) and taking input in a formula as used for \code{\link[graphics]{boxplot}}.
 #'
 #' @param x for specifying data from which the boxplots are to be produced. Either a numeric vector, or a single list containing such vectors. Additional unnamed arguments specify further data as separate vectors (each corresponding to a component boxplot). NAs are allowed in the data.
-#' @param ... additional data vectors or formula parameters.
+#' @param ... additional data vectors or formula parameters. For the formula method, named arguments to be passed to the default method.
 #' @param formula a formula, such as y ~ grp, where y is a numeric vector of data values to be split into groups according to the grouping variable grp (usually a factor).
 #' @param data a data.frame (or list) from which the variables in formula should be taken.
+#' @param subset	an optional vector specifying a subset of observations to be used for plotting.
+#' @param drop,sep,lex.order defines groups to plot from formula, passed to  \code{\link[base]{split.default}}, see there.
 #' @param range a factor to calculate the upper/lower adjacent values
 #' @param h the height for the density estimator, if omit as explained in sm.density, h will be set to an optimum
 #' @param ylim y limits
@@ -38,6 +40,7 @@
 #' @param na.rm logical value indicating whether NA values should be stripped before the computation proceeds. Defaults to TRUE.
 #' @param side defaults to "both". Assigning "left" or "right" enables one sided plotting of violins. May be applied as a scalar across all groups.
 #' @param plotCentre defaults to "points", plotting a central point at the median. If "line" is given a median line is plotted (subject to side) alternatively.
+#' @param axes,frame.plot,panel.first,panel.last,asp,line,outer,adj,ann,ask,bg,bty,cin,col.axis,col.lab,col.main,col.sub,cra,crt,csi,cxy,din,err,family,fg,fig,fin,font,font.axis,font.lab,font.main,font.sub,lab,las,lend,lheight,ljoin,lmitre,mai,mar,mex,mfcol,mfg,mfrow,mgp,mkh,new,oma,omd,omi,page,pch,pin,plt,ps,pty,smo,srt,tck,tcl,usr,xlog,xaxp,xaxs,xaxt,xpd,yaxp,yaxs,ylbias Arguments to be passed to methods, such as graphical parameters (see \code{\link[graphics]{par}})).
 #' @keywords plot graphics violin
 #' @import sm
 #' @importFrom zoo rollmean
@@ -64,6 +67,21 @@
 #' vioplot(x, col="tomato", horizontal=TRUE, at=-4, add=TRUE,lty=2, rectCol="gray")
 #' vioplot(y, col="cyan", horizontal=FALSE, at=-4, add=TRUE,lty=2)
 #'
+#' # formula input
+#' data("iris")
+#' vioplot(Sepal.Length~Species, data = iris, main = "Sepal Length",
+#'         col=c("lightgreen", "lightblue", "palevioletred"))
+#' legend("topleft", legend=c("setosa", "versicolor", "virginica"),
+#'        fill=c("lightgreen", "lightblue", "palevioletred"), cex = 0.5)
+#'
+#' data("diamonds", package = "ggplot2")
+#' palette <- RColorBrewer::brewer.pal(9, "Pastel1")
+#' par(mfrow=c(3, 1))
+#' vioplot(price ~ cut, data = diamonds, las = 1, col = palette)
+#' vioplot(price ~ clarity, data = diamonds, las = 2, col = palette)
+#' vioplot(price ~ color, data = diamonds, las = 2, col = palette)
+#' par(mfrow=c(3, 1))
+#'
 #' #generate example data
 #' data_one <- rnorm(100)
 #' data_two <- rnorm(50, 1, 2)
@@ -128,24 +146,30 @@ vioplot <- function(x, ...) {
 #' @rdname vioplot
 #' @export
 vioplot.formula <-
-  function (formula, data = NULL, ..., names=NULL, na.action = NULL)
+  function (formula, data = NULL, ..., subset,  na.action = NULL,
+            add = FALSE, ann = !add, horizontal = FALSE,
+            xlab = mklab(y_var = horizontal), ylab = mklab(y_var = !horizontal), names=NULL,
+            drop = FALSE, sep = ".", lex.order = FALSE)
   {
     if (missing(formula) || (length(formula) != 3L))
       stop("'formula' missing or incorrect")
+    if (missing(xlab) || missing(ylab))
+      mklab <- function(y_var) if (y_var)
+        names(mf)[response]
+    else paste(names(mf)[-response], collapse = " : ")
     m <- match.call(expand.dots = FALSE)
     if (is.matrix(eval(m$data, parent.frame())))
-      m$data <- data.frame(as.numeric(data))
-    m$... <- NULL
+      m$data <- as.data.frame(data)
+    m$... <- m$drop <- m$sep <- m$lex.order <- NULL
+    m$xlab <- m$ylab <- m$add <- m$ann <- m$horizontal <- NULL
     m$names <- NULL
     m$na.action <- na.action
-    m[[1L]] <- quote(stats::model.frame)
+    m[[1L]] <- quote(stats::model.frame.default)
     mf <- eval(m, parent.frame())
     response <- attr(attr(mf, "terms"), "response")
-    datas <- split(mf[[response]], mf[-response])
-    with(datas, expr = {
-        if(is.null(names)) names <- names(datas)
-        eval(parse(text=paste("vioplot(", paste(names(datas), collapse = ", "), ", names = names, ...)")))
-    })
+    vioplot(split(mf[[response]], mf[-response], drop = drop,
+            sep = sep, lex.order = lex.order), xlab = xlab, ylab = ylab, names = names,
+            add = add, ann = ann, horizontal = horizontal, ...)
   }
 
 
@@ -153,36 +177,58 @@ vioplot.formula <-
 #' @export
 vioplot.default <-
   function (x, ..., data = NULL, range = 1.5, h = NULL, ylim = NULL, names = NULL,
-            horizontal = FALSE, col = "grey50", border = "black", lty = 1,
-            lwd = 1, rectCol = "black", lineCol = "black", pchMed = 19, colMed = "white", colMed2 = "grey 75",
+            horizontal = FALSE, col = "grey50", border = par()$fg, lty = 1,
+            lwd = 1, rectCol = par()$fg, lineCol = par()$fg, pchMed = 19, colMed = "white", colMed2 = "grey 75",
             at, add = FALSE, wex = 1, drawRect = TRUE, areaEqual=FALSE,
-            main=NA, sub=NA, xlab=NA, ylab=NA, cex=1, cex.axis=1, cex.names=NULL, cex.lab=1, cex.main=1, cex.sub=1,
-            yaxt="s", ylog=FALSE, log="", logLab=c(1,2,5),
+            axes = TRUE, frame.plot = axes, panel.first = NULL, panel.last = NULL, asp = NA,
+            main="", sub="", xlab=NA, ylab=NA, line = NA, outer = FALSE,
+            xlog = NA, ylog=NA, adj=NA, ann = NA, ask=NA, bg=NA, bty=NA,
+            cex=NA, cex.axis=NA, cex.lab=NA, cex.main=NA, cex.names=NULL, cex.sub=NA,
+            cin=NA, col.axis=NA, col.lab=NA, col.main=NA, col.sub=NA,
+            cra=NA, crt=NA, csi=NA,cxy=NA, din=NA, err=NA, family=NA, fg=NA,
+            fig=NA, fin=NA, font=NA, font.axis=NA, font.lab=NA, font.main=NA, font.sub=NA,
+            lab=NA, las=NA, lend=NA, lheight=NA, ljoin=NA, lmitre=NA, mai=NA, mar=NA, mex=NA,
+            mfcol=NA, mfg=NA, mfrow=NA, mgp=NA, mkh=NA, new=NA, oma=NA, omd=NA, omi=NA,
+            page=NA, pch=NA, pin=NA, plt=NA, ps=NA, pty=NA, smo=NA, srt=NA, tck=NA, tcl=NA,
+            usr=NA, xaxp=NA, xaxs=NA, xaxt=NA, xpd=NA, yaxp=NA, yaxs=NA, yaxt=NA, ylbias=NA,
+            log="", logLab=c(1,2,5),
             na.action = NULL, na.rm = T, side = "both", plotCentre = "point")
   {
+    #assign graphical parameters if not given
+    for(ii in 1:length(names(par()))){
+      if(is.na(get(names(par())[ii])[1])) assign(names(par()[ii]), unlist(par()[[ii]]))
+    }
     if(!is.list(x)){
       datas <- list(x, ...)
     } else{
       datas<-lapply(x, unlist)
+      if(is.null(names)){
+        names <- names(datas)
+      }
     }
     if(is.character(log)) if("y" %in% unlist(strsplit(log, ""))) log <- TRUE
-    if(log == TRUE | ylog == TRUE) ylog <- TRUE
+    log <- ifelse(log == TRUE, "y", "")
+    if(log == 'x' | log == 'xy' | xlog == TRUE){
+      if(horizontal | log == "xy"){
+        log <- TRUE
+      } else {
+        log <- FALSE
+        ylog <- FALSE
+      }
+      xlog <- FALSE
+    }
+    if(log == TRUE | ylog == TRUE){
+      ylog <- TRUE
+      log <- "y"
+    }
     if(ylog){
       #check data is compatible with log scale
       if(all(unlist(datas) <= 0)){
         ylog <- FALSE
         warning("log scale cannot be used with non-positive data")
       } else {
-        #create axis labels
-        log_axis <- as.vector(outer(1:9, 10^(floor(log(min(unlist(datas)), 10)):ceiling(log(max(unlist(datas)), 10)))))
-        log_axis <- log_axis[log_axis < max(unlist(datas))]
-        log_axis_label <- ifelse(sapply(strsplit(as.character(log_axis), split=""), function(xx) any(logLab %in% xx)), log_axis, "")
-        #log_axis_label <- ifelse(log(log_axis, 10) %% 1 ==0, log_axis, "")
-        #log_axis <- sort(c(1, 10^c(seq(-10,10)), 2*10^c(seq(-10,10)), 5*10^c(seq(-10,10))))
-        #log_axis_label <- log_axis
-
         #log-scale data
-        datas <- lapply(datas, function(x) log(unlist(x)))
+        datas <- datas #lapply(datas, function(x) log(unlist(x)))
       }
     }
     if(is.null(na.action)) na.action <- na.omit
@@ -286,120 +332,113 @@ vioplot.default <-
       label <- names
     }
     boxwidth <- 0.05 * ifelse(length(boxwex)>1, boxwex[i], boxwex)
-    if (!add)
+    if (!add){
       plot.new()
+      if(!horizontal){
+        plot.window(xlim, ylim, log = log, asp = asp, bty = bty, cex = cex, xaxs = xaxs, yaxs = yaxs, lab = lab, mai = mai, mar = mar, mex = mex, mfcol = mfcol, mfrow = mfrow, mfg = mfg, xlog = xlog, ylog = ylog)
+      } else {
+        plot.window(ylim, xlim, log = ifelse(log == "y", "x", ""), asp = asp, bty = bty, cex = cex, xaxs = xaxs, yaxs = yaxs, lab = lab, mai = mai, mar = mar, mex = mex, mfcol = mfcol, mfrow = mfrow, mfg = mfg, xlog = ylog, ylog = xlog)
+      }
+          }
+    panel.first
     if (!horizontal) {
       if (!add) {
-        plot.window(xlim = xlim, ylim = ylim)
+        plot.window(xlim, ylim, log = log, asp = asp, bty = bty, cex = cex, xaxs = xaxs, yaxs = yaxs, lab = lab, mai = mai, mar = mar, mex = mex, mfcol = mfcol, mfrow = mfrow, mfg = mfg, xlog = xlog, ylog = ylog)
+        xaxp <- par()$xaxp
+        yaxp <- par()$yaxp
         if(yaxt !="n"){
           if(ylog){
             #log_axis_label <- log_axis_label[log_axis >= exp(par("usr")[3])]
             #log_axis <- log_axis[log_axis >= exp(par("usr")[3])]
             #log_axis_label <- log_axis_label[log_axis <= exp(par("usr")[4])]
             #log_axis <- log_axis[log_axis <= exp(par("usr")[4])]
-            axis(2, at=log(log_axis), labels=log_axis_label, cex.axis=cex*cex.axis)
+            Axis(unlist(datas), side = 2, cex.axis = cex.axis, col.axis = col.axis, font.axis = font.axis, mgp = mgp, tck = tck, tcl = tcl, las = las) # xaxp = xaxp, yaxp = yaxp disabled for log
+            if(is.null(cex.names)) cex.names <- cex.axis
+            Axis(1:length(datas), at = at, labels = label, side = 1, cex.axis = cex.names, col.axis = col.axis, font.axis = font.axis, mgp = mgp, tck = tck, tcl = tcl, las = las) # xaxp = xaxp, yaxp = yaxp disabled for log
           } else {
-            axis(2, cex.axis=cex*cex.axis)
+            Axis(unlist(datas), side = 2, cex.axis = cex.axis, col.axis = col.axis, font.axis = font.axis, mgp = mgp, yaxp = yaxp, tck = tck, tcl = tcl, las = las)
+            if(is.null(cex.names)) cex.names <- cex.axis
+            Axis(1:length(datas), at = at, labels = label, side = 1, cex.axis = cex.names, col.axis = col.axis, font.axis = font.axis, mgp = mgp, xaxp = xaxp, tck = tck, tcl = tcl, las = las)
           }
         }
-        if(is.null(cex.names)) cex.names <- cex.axis
-        axis(1, at = at, labels = label, cex.axis=cex*cex.names)
       }
-      box()
+      if (frame.plot) {
+       box(lty = lty, lwd = lwd)
+      }
       for (i in 1:n) {
         polygon(c(at[i] - radj*height[[i]], rev(at[i] + ladj*height[[i]])),
                 c(base[[i]], rev(base[[i]])), col = ifelse(length(col)>1, col[i], col), border = ifelse(length(border)>1, border[i], border),
-                lty = lty, lwd = lwd)
+                lty = lty, lwd = lwd, xpd = xpd, lend = lend, ljoin = ljoin, lmitre = lmitre)
         if (drawRect) {
           lines(at[c(i, i)], c(lower[i], upper[i]), lwd = lwd,
-                lty = lty, col = ifelse(length(lineCol)>1, lineCol[i], lineCol))
+                lty = lty, col = ifelse(length(lineCol)>1, lineCol[i], lineCol), lend = lend, ljoin = ljoin, lmitre = lmitre)
           rect(at[i] - radj*ifelse(length(boxwidth)>1, boxwidth[i], boxwidth)/2, q1[i], at[i] + ladj*ifelse(length(boxwidth)>1, boxwidth[i], boxwidth)/2,
-               q3[i], col = ifelse(length(rectCol)>1, rectCol[i], rectCol), border = ifelse(length(lineCol)>1, lineCol[i], lineCol))
+               q3[i], col = ifelse(length(rectCol)>1, rectCol[i], rectCol), border = ifelse(length(lineCol)>1, lineCol[i], lineCol), xpd = xpd, lend = lend, ljoin = ljoin, lmitre = lmitre)
           if(plotCentre == "line"){
             lines(x = c(at[i] - radj*med.dens[i],
                         at[i],
                         at[i] + ladj*med.dens[i]),
                   y = rep(med[i],3))
           } else {
-            points(at[i], med[i], pch = ifelse(length(pchMed)>1, pchMed[i], pchMed), col = ifelse(length(colMed)>1, colMed[i], colMed), bg = ifelse(length(colMed2)>1, colMed2[i], colMed2))
+            points(at[i], med[i], pch = ifelse(length(pchMed)>1, pchMed[i], pchMed), col = ifelse(length(colMed)>1, colMed[i], colMed), bg = ifelse(length(colMed2)>1, colMed2[i], colMed2), cex = cex, lwd = lwd, lty = lty)
           }
         }
       }
     }
     else {
+      if(log == "y" || ylog == TRUE){
+        log <- "x"
+        xlog <- TRUE
+        ylog <- FALSE
+      }
       if (!add) {
-        plot.window(xlim = ylim, ylim = xlim)
-        axis(1)
+        plot.window(ylim, xlim, log = log, asp = asp, bty = bty, cex = cex, xaxs = xaxs, yaxs = yaxs, lab = lab, mai = mai, mar = mar, mex = mex, mfcol = mfcol, mfrow = mfrow, mfg = mfg, xlog = xlog, ylog = ylog)
+        xaxp <- par()$xaxp
+        yaxp <- par()$yaxp
         if(yaxt !="n"){
-          if(ylog){
+          if(xlog){
             #log_axis_label <- log_axis_label[log_axis >= exp(par("usr")[3])]
             #log_axis <- log_axis[log_axis >= exp(par("usr")[3])]
             #log_axis_label <- log_axis_label[log_axis <= exp(par("usr")[4])]
             #log_axis <- log_axis[log_axis <= exp(par("usr")[4])]
-            axis(2, at=log(log_axis), labels=log_axis_label)
+            Axis(unlist(datas), side = 1, cex.axis = cex.names, col.axis = col.axis, font.axis = font.axis, mgp = mgp, tck = tck, tcl = tcl, las = las) # xaxp = xaxp, yaxp = yaxp disabled for log
+            if(is.null(cex.names)) cex.names <- cex.axis
+            Axis(1:length(datas), at = at, labels = label, side = 2, cex.axis = cex.axis, col.axis = col.axis, font.axis = font.axis, mgp = mgp, tck = tck, tcl = tcl, las = las) # xaxp = xaxp, yaxp = yaxp disabled for log
           } else {
-            axis(2, at = at, labels = label)
+            Axis(unlist(datas), side = 1, cex.axis = cex.names, col.axis = col.axis, font.axis = font.axis, mgp = mgp, xaxp = xaxp, tck = tck, tcl = tcl, las = las)
+            if(is.null(cex.names)) cex.names <- cex.axis
+            Axis(1:length(datas), at = at, labels = label, side = 2, cex.axis = cex.axis, col.axis = col.axis, font.axis = font.axis, mgp = mgp, yaxp = yaxp, tck = tck, tcl = tcl, las = las)
           }
         }
       }
-      box()
+      if (frame.plot) {
+        box(lty = lty, lwd = lwd)
+      }
       for (i in 1:n) {
         polygon(c(base[[i]], rev(base[[i]])), c(at[i] - radj*height[[i]],
                                                 rev(at[i] + ladj*height[[i]])), col = ifelse(length(col)>1, col[i], col), border = ifelse(length(border)>1, border[i], border),
-                lty = lty, lwd = lwd)
+                lty = lty, lwd = lwd, xpd = xpd, lend = lend, ljoin = ljoin, lmitre = lmitre)
         if (drawRect) {
           lines(c(lower[i], upper[i]), at[c(i, i)], lwd = lwd,
-                lty = lty, col = ifelse(length(lineCol)>1, lineCol[i], lineCol))
+                lty = lty, col = ifelse(length(lineCol)>1, lineCol[i], lineCol), lend = lend, ljoin = ljoin, lmitre = lmitre)
           rect(q1[i], at[i] - radj*ifelse(length(boxwidth)>1, boxwidth[i], boxwidth)/2, q3[i], at[i] +
-                 ladj*ifelse(length(boxwidth)>1, boxwidth[i], boxwidth)/2, col = ifelse(length(rectCol)>1, rectCol[i], rectCol), border = ifelse(length(lineCol)>1, lineCol[i], lineCol))
+                 ladj*ifelse(length(boxwidth)>1, boxwidth[i], boxwidth)/2, col = ifelse(length(rectCol)>1, rectCol[i], rectCol), border = ifelse(length(lineCol)>1, lineCol[i], lineCol), xpd = xpd, lend = lend, ljoin = ljoin, lmitre = lmitre)
           if(plotCentre == "line"){
             lines(y = c(at[i] - radj*med.dens[i],
                         at[i],
                         at[i] + ladj*med.dens[i]),
                   x = rep(med[i],3))
           } else {
-            points(med[i], at[i], pch = ifelse(length(pchMed)>1, pchMed[i], pchMed), col = ifelse(length(colMed)>1, colMed[i], colMed), , bg = ifelse(length(colMed2)>1, colMed2[i], colMed2))
+            points(med[i], at[i], pch = ifelse(length(pchMed)>1, pchMed[i], pchMed), col = ifelse(length(colMed)>1, colMed[i], colMed), , bg = ifelse(length(colMed2)>1, colMed2[i], colMed2), cex = cex, lwd = lwd, lty = lty)
           }
         }
       }
     }
+    panel.last
+    if (ann) {
+        title(main = main, sub = sub, xlab = xlab, ylab = ylab, line = line, outer = outer, xpd = xpd, cex.main = cex.main, col.main = col.main, font.main = font.main)
+    }
     invisible(list(upper = upper, lower = lower, median = med,
                    q1 = q1, q3 = q3))
-    if(is.na(xlab)==F){
-      if(is.na(ylab)==F){
-        if(is.na(main)==F){
-          if(is.na(sub)==T){
-            title(main=main, ylab=ylab, xlab=xlab, cex.main=cex*cex.main, cex.lab=cex*cex.lab)
-          } else{
-            title(main=main, sub=sub, ylab=ylab, xlab=xlab, cex.main=cex*cex.main, cex.sub=cex*cex.sub, cex.lab=cex*cex.lab)
-          }
-        }
-      } else{
-        if(is.na(main)==F){
-          if(is.na(sub)==T){
-            title(main=main, xlab=xlab, cex.main=cex*cex.main, cex.lab=cex*cex.lab)
-          } else{
-            title(main=main, sub=sub, xlab=xlab, cex.main=cex*cex.main, cex.sub=cex*cex.sub, cex.lab=cex*cex.lab)
-          }
-        }
-      }
-    } else {
-      if(is.na(ylab)==F){
-        if(is.na(main)==F){
-          if(is.na(sub)==T){
-            title(main=main, ylab=ylab, cex.main=cex*cex.main, cex.lab=cex*cex.lab)
-          } else{
-            title(main=main, sub=sub, ylab=ylab, cex.main=cex*cex.main, cex.sub=cex*cex.sub, cex.lab=cex*cex.lab)
-          }
-        }
-      } else{
-        if(is.na(main)==F){
-          if(is.na(sub)==T){
-            title(main=main, cex.main=cex*cex.main)
-          } else{
-            title(main=main, sub=sub, cex.main=cex*cex.main, cex.sub=cex*cex.sub)
-          }
-        }
-      }
-    }
   }
+
